#include "EEPROM.h"

uint8_t programFlash(uint32_t address, uint16_t data)
{
    // Wait for FPEC to be ready
    while(((FPEC->SR >> 0) & 0x01) == 1);
    
    // Check whether the flash is locked
    if(((FPEC->CR >> 7) & 0x01) == 1)
    {
        // Unlocking the flash memory
        FPEC->KEYR = 0x45670123UL;
        FPEC->KEYR = 0xCDEF89ABUL;
    }
    
    // Enable standard programming
    FPEC->CR &= ~(0x01 << 6);   // STRT = 0
    FPEC->CR &= ~(0x01 << 1);   // PER = 0
    FPEC->CR |= (0x01 << 0);    // PG = 1
    
    // Perform half-word write operation at desired address
    *((uint16_t*)(address & 0xFFFFFFFEUL)) = data;
    
    // Wait for FPEC to be ready
    while(((FPEC->SR >> 0) & 0x01) == 1);
    // Wait for EOP bit to be cleared
    while(((FPEC->SR >> 5) & 0x01) == 0);
    
    FPEC->CR &= ~(0x01 << 6);   // STRT = 0
    FPEC->CR &= ~(0x01 << 1);   // PER = 0
    FPEC->CR &= ~(0x01 << 0);   // PG = 0
    
    if(((FPEC->SR >> 2) & 0x01) == 1)
    {
        while(1)
        {
            wait_cycles(100000);
            setPinState(PORTB, 15, 1);
            wait_cycles(100000);
            setPinState(PORTB, 15, 0);
        }
    }
    
    if(*((uint16_t*)(address & 0xFFFFFFFEUL)) == data)
    {
        return 0;
    }
    else
    {
        return 1;
    }
}

uint16_t readFlash(uint32_t address)
{
    return *((uint16_t*)(address & 0xFFFFFFFEUL));
}

uint8_t pageEraseFlash(uint32_t address)
{
    // Wait for FPEC to be ready
    while(((FPEC->SR >> 0) & 0x01) == 1);
    
    // Check whether the flash is locked
    if(((FPEC->CR >> 7) & 0x01) == 1)
    {
        // Unlocking the flash memory
        FPEC->KEYR = 0x45670123UL;
        FPEC->KEYR = 0xCDEF89ABUL;
    }
    
    // FLASH_FLAG_EOP|FLASH_FLAG_PGERR|FLASH_FLAG_WRPRTERR
    //FPEC->SR |= (0x01 << 5);
    //FPEC->SR |= (0x01 << 4);
    //FPEC->SR |= (0x01 << 2);
    
    // Enable page erase
    FPEC->CR &= ~(0x01 << 6);   // STRT = 0
    FPEC->CR |= (0x01 << 1);    // PER = 1
    FPEC->CR &= ~(0x01 << 0);   // PG = 0
    
    // Set address (must be within the page to erase)
    FPEC->AR = address;
    
    // Perform page erase
    FPEC->CR |= (0x01 << 6);    // STRT = 1
    
    // Wait for FPEC to be ready
    while(((FPEC->SR >> 0) & 0x01) == 1);
    // Wait for EOP bit to be cleared
    while(((FPEC->SR >> 5) & 0x01) == 0);
    
    FPEC->CR &= ~(0x01 << 6);   // STRT = 0
    FPEC->CR &= ~(0x01 << 1);   // PER = 0
    FPEC->CR &= ~(0x01 << 0);   // PG = 0
    
    if(((FPEC->SR >> 2) & 0x01) == 1)
    {
        while(1)
        {
            wait_cycles(100000);
            setPinState(PORTB, 12, 1);
            wait_cycles(100000);
            setPinState(PORTB, 12, 0);
        }
    }
    
    // Check if the page has been erased correctly be reading all addresses within the page
    uint8_t flag = 0;
    for(uint32_t offset = 0; offset < PAGE_SIZE; offset += 2)
    {
        uint32_t c_addr = (address & 0xFFFFFC00UL) + offset;
        
        if((*(uint16_t*)c_addr) != 0xFFFFUL)
        {
            flag = 1;
        }
    }
    
    return flag;
}
