#include "STM32F100.h"

volatile RCC_T *RCC = (RCC_T*)0x40021000;
volatile GPIO_T *GPIOA = (GPIO_T*)0x40010800;
volatile GPIO_T *GPIOB = (GPIO_T*)0x40010C00;
volatile GPIO_T *GPIOC = (GPIO_T*)0x40011000;
volatile GPIO_T *GPIOD = (GPIO_T*)0x40011400;
volatile GPIO_T *GPIOE = (GPIO_T*)0x40011800;
volatile AFIO_T *AFIO = (AFIO_T*)0x40010000;
volatile NVIC_T *NVIC = (NVIC_T*)0xE000E100;
volatile uint32_t *NVIC_STIR = (uint32_t*)0xE00EF00;
volatile TIM_2_5_T *TIM2 = (TIM_2_5_T*)0x40000000;
volatile TIM_15_T *TIM15 = (TIM_15_T*)0x40014000;
volatile FPEC_T *FPEC = (FPEC_T*)0x40022000;

void wait_cycles(int N)
{
    volatile static int c = 0;
    for(int i = 0; i < N; i++)
    {
        c++;
    }
}

void setup_clock_HSI(uint8_t mult, uint8_t prediv)
{
    RCC->CR |= (0x01 << 0);    // HSI on
    
    // Wait for HSI to be ready
    while(((RCC->CR >> 1) & 0x01) == 0)
    {
        ;
    }
    
    // Configure PLL
    RCC->CFGR2 |= (((prediv - 1) & 0x0F) << 0);    // Set PREDIV factor
    RCC->CFGR |= (((mult - 2) & 0x0F) << 18);    // Set Multiplication factor
    RCC->CFGR &= ~(0x01 << 16);    // Set PLL clock source
    //RCC->CFGR |= (0x02 << 0);    // Use PLL as system clock source
    
    RCC->CR |= (0x01 << 24);    // PLL on
    
    // Wait for PLL to be ready
    while(((RCC->CR >> 25) & 0x01) == 0)
    {
        ;
    }
    
    RCC->CFGR |= (0x02 << 0);    // Use PLL as system clock source
}

void setup_clock_HSE(uint8_t mult, uint8_t prediv)
{
    RCC->CR |= (0x01 << 16);    // HSE on
    
    // Wait for HSE to be ready
    while(((RCC->CR >> 17) & 0x01) == 0)
    {
        ;
    }
    
    // Configure PLL
    RCC->CFGR2 |= (((prediv - 1) & 0x0F) << 0);    // Set PREDIV factor
    RCC->CFGR |= (((mult - 2) & 0x0F) << 18);    // Set Multiplication factor
    RCC->CFGR |= (0x01 << 16);    // Set PLL clock source
    RCC->CFGR |= (0x02 << 0);    // Use PLL as system clock source
    
    RCC->CR |= (0x01 << 24);    // PLL on
    
    // Wait for PLL to be ready
    while(((RCC->CR >> 25) & 0x01) == 0)
    {
        ;
    }
    
    RCC->CFGR |= (0x02 << 0);    // Use PLL as system clock source
}

void initGPIO()
{
    // Enable Port A, B, C and D
    RCC->APB2ENR |= (0x01 << 5) | (0x01 << 4) | (0x01 << 3) | (0x01 << 2);
    
    // Attention, this will disable SWJ so programming the device becomes difficult!
    // PA13, PA14 and PA15 as normal GPIO
    //RCC->APB2ENR |= (0x01 << 0);
    //AFIO->MAPR |= (0x04 << 24);
}

uint8_t getPinState(uint8_t port, uint8_t pin)
{
    uint8_t ret = -1;
    switch(port)
    {
        case PORTA:
            ret = ((GPIOA->IDR) >> pin) & 0x01;
        break;
        case PORTB:
            ret = ((GPIOB->IDR) >> pin) & 0x01;
        break;
        case PORTC:
            ret = ((GPIOC->IDR) >> pin) & 0x01;
        break;
        case PORTD:
            ret = ((GPIOD->IDR) >> pin) & 0x01;
        break;
    }
    
    return ret;
}

void setPinState(uint8_t port, uint8_t pin, uint8_t state)
{
    switch(port)
    {
        case PORTA:
            if(state == 1)
            {
                GPIOA->BSRR = (0x01 << pin);
            }
            else
            {
                GPIOA->BSRR = (0x01 << (pin + 16));
            }
        break;
        case PORTB:
            if(state == 1)
            {
                GPIOB->BSRR = (0x01 << pin);
            }
            else
            {
                GPIOB->BSRR = (0x01 << (pin + 16));
            }
        break;
        case PORTC:
            if(state == 1)
            {
                GPIOC->BSRR = (0x01 << pin);
            }
            else
            {
                GPIOC->BSRR = (0x01 << (pin + 16));
            }
        break;
        case PORTD:
            if(state == 1)
            {
                GPIOD->BSRR = (0x01 << pin);
            }
            else
            {
                GPIOD->BSRR = (0x01 << (pin + 16));
            }
        break;
    }
}

void setPinDir(uint8_t port, uint8_t pin, uint8_t dir)
{
    setPinDirExt(port, pin, dir, GPIO_PULLUP);
}

void setPinDirExt(uint8_t port, uint8_t pin, uint8_t dir, uint8_t pullup)
{
    volatile GPIO_T *gpio_reg;
    switch(port)
    {
        case PORTA:
            gpio_reg = GPIOA;
        break;
        case PORTB:
            gpio_reg = GPIOB;
        break;
        case PORTC:
            gpio_reg = GPIOC;
        break;
        case PORTD:
            gpio_reg = GPIOD;
        break;
        default:
            gpio_reg = 0;
        break;
    }
    
    if(dir == GPIO_OUTPUT)
    {
        if((pin >= 0) && (pin <= 7))
        {
            gpio_reg->CRL &= ~(0x03 << (4 * pin + 2)) & ~(0x03 << (4 * pin));
            gpio_reg->CRL |= (0x00 << (4 * pin + 2)) | (0x03 << (4 * pin));
        }
        else
        {
            gpio_reg->CRH &= ~(0x03 << (4 * (pin - 8) + 2)) & ~(0x03 << (4 * (pin - 8)));
            gpio_reg->CRH |= (0x00 << (4 * (pin - 8) + 2)) | (0x03 << (4 * (pin - 8)));
        }
    }
    else
    {
        if((pin >= 0) && (pin <= 7))
        {
            gpio_reg->CRL &= ~(0x03 << (4 * pin + 2)) & ~(0x03 << (4 * pin));
            if((pullup == GPIO_PULLUP) || (pullup == GPIO_PULLDOWN))
            {
                gpio_reg->CRL |= (0x02 << (4 * pin + 2)) | (0x00 << (4 * pin));
            }
            else
            {
                gpio_reg->CRL |= (0x01 << (4 * pin + 2)) | (0x00 << (4 * pin));
            }
        }
        else
        {
            gpio_reg->CRH &= ~(0x03 << (4 * (pin - 8) + 2)) & ~(0x03 << (4 * (pin - 8)));
            if((pullup == GPIO_PULLUP) || (pullup == GPIO_PULLDOWN))
            {
                gpio_reg->CRH |= (0x02 << (4 * (pin - 8) + 2)) | (0x00 << (4 * (pin - 8)));
            }
            else
            {
                gpio_reg->CRH |= (0x01 << (4 * (pin - 8) + 2)) | (0x00 << (4 * (pin - 8)));
            }
        }
        
        if(pullup == GPIO_PULLUP)
        {
            gpio_reg->ODR |= (1 << pin);
        }
    }
}
